@@ -1,16 +1,17 @@
-#Copyright ReportLab Europe Ltd. 2000-2012
+#Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
 """Tests for reportlab.lib.utils
 """
-__version__=''' $Id$ '''
+__version__='3.3.0'
 from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, printLocation
 setOutDir(__name__)
 import os, time, sys
 import reportlab
+from reportlab import rl_config
 import unittest
 from reportlab.lib import colors
 from reportlab.lib.utils import recursiveImport, recursiveGetAttr, recursiveSetAttr, rl_isfile, \
-                                isCompactDistro, isPy3
+                                isCompactDistro, isPy3, isPyPy, TimeStamp, rl_get_module
 
 def _rel_open_and_read(fn):
     from reportlab.lib.utils import open_and_read
@@ -24,26 +25,26 @@ def _rel_open_and_read(fn):
 
 class ImporterTestCase(unittest.TestCase):
     "Test import utilities"
-    count = 0
 
-    def setUp(self):
+    @classmethod
+    def setUpClass(cls):
         from reportlab.lib.utils import get_rl_tempdir
-        s = repr(int(time.time())) + repr(self.count)
-        self.__class__.count += 1
-        self._tempdir = get_rl_tempdir('reportlab_test','tmp_%s' % s)
-        if not os.path.isdir(self._tempdir):
-            os.makedirs(self._tempdir,0o700)
-        _testmodulename = os.path.join(self._tempdir,'test_module_%s.py' % s)
-        f = open(_testmodulename,'w')
-        f.write('__all__=[]\n')
-        f.close()
+        cls._value = float(repr(time.time()))
+        s = int(cls._value)
+        cls._tempdir = get_rl_tempdir('reportlab_test','tmp_%s' % s)
+        if not os.path.isdir(cls._tempdir):
+            os.makedirs(cls._tempdir,0o700)
+        _testmodulename = os.path.join(cls._tempdir,'test_module_%s.py' % s)
+        with open(_testmodulename,'w') as f:
+            f.write('__all__=[]\nvalue=%s\n' % repr(cls._value))
         if sys.platform=='darwin' and isPy3:
             time.sleep(0.3)
-        self._testmodulename = os.path.splitext(os.path.basename(_testmodulename))[0]
+        cls._testmodulename = os.path.splitext(os.path.basename(_testmodulename))[0]
 
-    def tearDown(self):
+    @classmethod
+    def tearDownClass(cls):
         from shutil import rmtree
-        rmtree(self._tempdir,1)
+        rmtree(cls._tempdir,1)
 
     def test1(self):
         "try stuff known to be in the path"
@@ -139,7 +140,7 @@ class ImporterTestCase(unittest.TestCase):
             self.fail("Imported a nonexistent module")
         except ImportError as e:
             self.assertIn('reportlab.pdfgen.brush',str(e))
-            
+
         try:
             m1 = recursiveImport('totally.non.existent')
             self.fail("Imported a nonexistent module")
@@ -151,7 +152,60 @@ class ImporterTestCase(unittest.TestCase):
             m1 = recursiveImport('unimportable')
             self.fail("Imported a buggy module")
         except Exception as e:
-            self.assertIn(reportlab.isPy3 and 'division by zero' or 'integer division or modulo by zero',str(e))
+            self.assertIn(("integer division by zeroException raised while importing 'unimportable': integer division by zero"
+                            if isPyPy
+                            else ('division by zero' if isPy3
+                                 else 'integer division or modulo by zero'))
+                        ,str(e))
+    def test14(self):
+        "test the TimeStamp behaviour"
+        oinvariant = rl_config.invariant
+        sden = 'SOURCE_DATE_EPOCH'
+        if sden in os.environ:
+            sde = os.environ[sden]
+            del os.environ[sden]
+        else:
+            sde = self
+
+        try:
+            rl_config.invariant = False
+            t = time.time()
+            ts = TimeStamp()
+            self.assertTrue(abs(t-ts.t)<1)
+            ts = TimeStamp(invariant=True)
+            self.assertEqual(ts.t,946684800.0)
+            self.assertEqual(ts.YMDhms,(2000, 1, 1, 0, 0, 0))
+            self.assertEqual(ts.tzname,'UTC')
+            os.environ[sden] = '1490003100'
+            ts = TimeStamp(invariant=True)  #debian variable takes precedence here
+            self.assertEqual(ts.t,1490003100)
+            self.assertEqual(ts.YMDhms,(2017, 3, 20, 9, 45, 0))
+            self.assertEqual(ts.tzname,'UTC')
+            rl_config.invariant = True
+            ts = TimeStamp()                #still takes precedence
+            self.assertEqual(ts.t,1490003100)
+            self.assertEqual(ts.YMDhms,(2017, 3, 20, 9, 45, 0))
+            self.assertEqual(ts.tzname,'UTC')
+            del os.environ[sden]
+            ts = TimeStamp()                #now rl_config takes precedence
+            self.assertEqual(ts.t,946684800.0)
+            self.assertEqual(ts.YMDhms,(2000, 1, 1, 0, 0, 0))
+            self.assertEqual(ts.tzname,'UTC')
+        finally:
+            if sde is not self:
+                os.environ[sden] = sde
+            rl_config.invariant = oinvariant
+
+    def test15(self):
+        m = rl_get_module(self._testmodulename,self._tempdir)
+        self.assertEqual(self._value,m.value)
+
+    def test16(self):
+        from reportlab.lib import fontfinder
+        ff = fontfinder.FontFinder(useCache=False,recur=True)
+        ff.addDirectories(rl_config.T1SearchPath + rl_config.TTFSearchPath)
+        ff.search()
+        ff.getFamilyNames()
 
 def makeSuite():
     return makeSuiteForClasses(ImporterTestCase)
